from bgpy.as_graphs.base import ASGraph


class CAIDAASGraph(ASGraph):
    """For now this is just a copy of the AS Graph

    Since CAIDA is the only AS Graph we currently support
    but that could change in the future
    """

    pass
